==> dwf-deploy-2.0.3.tgz: package/lib/dwf/object/type/taskChain.js <==
'use strict';

const coAsyncLib = require('async');

const coLoggerLib = require('../../../logger.js');
const coPathsLib = require('../../../paths.js');
const coUtils = require('../../../utils.js');

const coAbstractLib = require('./abstract.js');

const coRequestLib = require('../../request.js');
const Result = require('../../result.js').Result;
const coValidateLib = require('../../validate.js');
const coRepositoryLib = require('../../repository.js');
const coDwfAbstract = require('../../abstract.js');

const csNamespace = 'namespace';
const csTaskChainId = 'taskChainId';

const cfGetProvider = function (aoProviders, isExtensionFile) {
  let loProviderExt = {};
  aoProviders.forEach(function (ioProvider) { // Check - ioProvider schema validation
    let laoExtentions = ioProvider.extensions;
    laoExtentions.forEach(function(ioExtension) {
      let lsExtensionName = ioExtension.name;
      let lsExtensionRoute = ioExtension.route;
      if (isExtensionFile === '.' + lsExtensionName) {
        loProviderExt.org = ioProvider.org;
        loProviderExt.org_guid = ioProvider.org_guid;
        loProviderExt.space = ioProvider.space;
        loProviderExt.space_guid = ioProvider.space_guid;
        loProviderExt.name = ioProvider.name;
        loProviderExt.url = ioProvider.url + '/' + lsExtensionRoute;
        loProviderExt.extention = lsExtensionName;
        loProviderExt.route = lsExtensionRoute;
      }
    });
  });
  if (typeof loProviderExt.url === 'undefined' || loProviderExt.url === null) {
    throw new Error('Failed, because was not able to construct the url for the service provider for ' + isExtensionFile + ' files');
  }
  return loProviderExt;
};

const cfGetTaskChain4BatchPost = function(isFile, ioDir2Namespace) {
  let loData_TaskChain = coUtils.readJSONFile(isFile);
  let laParsedNamelessFile = coPathsLib.parseNamelessFile(isFile);
  let lsNamespace = ioDir2Namespace[laParsedNamelessFile.dir]; // Check
  if (typeof lsNamespace === 'undefined' || lsNamespace === null) {
    throw new Error('Failed, because the \'' + '\' namespace for ' + isFile + ' file is not supported');
  }
  let lsName = laParsedNamelessFile.name;
  if (typeof lsName === 'undefined' || lsName === null) { // Check - .dwftaskchain
    throw new Error('Failed, because the \'' + '\' id for ' + isFile + ' file is not supported');
  }

  loData_TaskChain[csNamespace] = lsNamespace;
  loData_TaskChain[csTaskChainId] = lsName;

  return loData_TaskChain;
};

const cfTaskChainPostBatch = function(isExt, iaoTaskChainPostBatch, ioProviderExt, ioCredential) {
  let lsProviderUrl = ioProviderExt.url;
  let loData = {
    uri: lsProviderUrl,
    json: true,
    method: 'POST',
    auth: {
      user: ioCredential.user,
      password: ioCredential.password,
      sendImmediately: true
    },
    body: iaoTaskChainPostBatch
  };
  let lfTask = function (ifCb) {
    coLoggerLib.trace(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ');
    let lfCb2Reuse4TaskChain = function(ioError, ioResponse, ioBody) {
      let loError;
      if (typeof ioError !== 'undefined' && ioError !== null) {
        loError = ioError;
      } else if (ioResponse.statusCode !== 200) {
        loError = new Error('Failed, because ' + loData.method + ' on ' + loData.uri + ' returned ' + ioResponse.statusCode +
                            ' as a response status code with body ' + JSON.stringify(ioBody));
      }
      if (typeof loError === 'undefined' || loError === null) {
        coLoggerLib.log(' * ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ok' + '(' + ioResponse.statusCode + ')');
      }
      ifCb(loError);
    };
    coRequestLib(loData, lfCb2Reuse4TaskChain);
  };
  let lafTask = [];
  lafTask.push(lfTask);
  return lafTask;
};

const cfTaskChainPostRepo = function(isExt, iaoResultItem, ioProviderExt, ioCredential, ioOption) {
  let loRepository = new coRepositoryLib.Repository(ioCredential, ioOption);
  let lafTaskWaterfall = loRepository.get_afTask2Update(isExt, iaoResultItem, ioProviderExt);
  let lafTask = [];
  lafTask.push(function(ifCb) {
    coAsyncLib.waterfall(lafTaskWaterfall, function(ioError, ioResult) {
      ifCb(ioError, ioResult);
    });
  });
  return lafTask;
};

const coType = {};
coType[coAbstractLib.csExtention] = '.dwftaskchain';
coType[coAbstractLib.cfPrepare] = undefined;
coType[coAbstractLib.cfCreateTasks] = function(iasFile, ioPreparedValue4Ext, ioCredential, ioOption) {
  coValidateLib(iasFile.length, 'number');
  coValidateLib(ioPreparedValue4Ext, 'object');
  coValidateLib(ioCredential, 'object');
  coValidateLib(ioOption, 'object');

  let loResult = new Result();
  let lsExt = coType[coAbstractLib.csExtention];
  // Creates the Task Chain for the Batch Post
  let lo_sFile2oTaskChainBatchPost = {};
  iasFile.forEach(function(isFile) {
    let lsFile = isFile;

    coLoggerLib.trace(' * ' + lsExt + ' - ' + lsFile + ' ... ');

    let loTaskChain4Batch = cfGetTaskChain4BatchPost(lsFile, ioPreparedValue4Ext);
    lo_sFile2oTaskChainBatchPost[isFile] = loTaskChain4Batch;

    let loItem = {};
    loItem[Result.csKey_AOI_nStatus] = Result.cnSuccess;
    loItem[Result.csKey_AOI_sFile] = lsFile;
    loItem[Result.csKey_AOI_sNamespace] = loTaskChain4Batch[csNamespace];
    loItem[Result.csKey_AOI_nID] = loTaskChain4Batch[csTaskChainId];
    loItem[Result.csKey_AOI_sDescription] = '';
    loResult.addItem(lsExt, loItem);

    coLoggerLib.log(' * ' + lsExt + ' - ' + lsFile + ' ... ok');
  });

  // Check (for two or more designtime files defining the same runtime object) and prepare
  let laoTaskChainPostBatch = [];
  let lo_sNamespace2o_sId2sFile = {}; // all for the check
  let lo_sNamespace2o_sId2asFile = {}; // errors
  iasFile.forEach(function(isFile) {
    let lsFile = isFile;

    let loTaskChainBatchPost = lo_sFile2oTaskChainBatchPost[lsFile];
    // Check
    let lsNamespace = loTaskChainBatchPost[csNamespace];
    let lsTaskChainId = loTaskChainBatchPost[csTaskChainId];
    let lo_sId2sFile = lo_sNamespace2o_sId2sFile[lsNamespace];
    if (typeof lo_sId2sFile === 'object') {
      let lsFileFirst = lo_sId2sFile[lsTaskChainId];
      if (typeof lsFileFirst === 'string') {
        if (typeof lo_sNamespace2o_sId2asFile[lsNamespace] === 'undefined') {
          lo_sNamespace2o_sId2asFile[lsNamespace] = {};
        }
        if (typeof lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId] === 'undefined') {
          lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId] = [lsFileFirst, lsFile];
        } else {
          lo_sNamespace2o_sId2asFile[lsNamespace][lsTaskChainId].push(lsFile);
        }
      } else {
        lo_sId2sFile[lsTaskChainId] = lsFile;
      }
    } else {
      lo_sNamespace2o_sId2sFile[lsNamespace] = {};
      lo_sNamespace2o_sId2sFile[lsNamespace][lsTaskChainId] = lsFile;
    }
    // Prepare
    laoTaskChainPostBatch.push(loTaskChainBatchPost);
  });
  if (Object.getOwnPropertyNames(lo_sNamespace2o_sId2asFile).length > 0) {
    throw new Error('Failed, because more than one designtime file claim to define the same runtime object, because of its namespace and id. ' +
                    JSON.stringify(lo_sNamespace2o_sId2asFile));
  }

  // Greates tasks for TOE and Repo
  let loProviderExt = cfGetProvider(ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1], lsExt);
  let loValue_afTask = [];
  let lafTaskToe = cfTaskChainPostBatch(lsExt, laoTaskChainPostBatch, loProviderExt, ioCredential);
  loValue_afTask = loValue_afTask.concat(lafTaskToe);
  let lafTaskRepo = cfTaskChainPostRepo(lsExt, loResult.get_aoItem(lsExt), loProviderExt, ioCredential, ioOption);
  loValue_afTask = loValue_afTask.concat(lafTaskRepo);
  loResult.addValue(lsExt, loValue_afTask);

  return loResult;
};

module.exports = coType;

==> dwf-deploy-2.2.0.tgz: package/lib/dwf/object/type/taskChain.js <==
'use strict';

const coAsyncLib = require('async');
const coLoggerLib = require('../../../logger.js');
const coAbstractLib = require('./abstract.js');
const coRequestLib = require('../../request.js');
const Result = require('../../result.js').Result;
const coValidateLib = require('../../validate.js');
const coRepositoryLib = require('../../repository.js');
const coDwfAbstract = require('../../abstract.js');

const cfGetProvider = function (aoProviders, isExtensionFile) {
  let loProviderExt = {};
  aoProviders.forEach(function (ioProvider) { // Check - ioProvider schema validation
    let laoExtentions = ioProvider.extensions;
    laoExtentions.forEach(function(ioExtension) {
      let lsExtensionName = ioExtension.name;
      let lsExtensionRoute = ioExtension.route;
      if (isExtensionFile === '.' + lsExtensionName) {
        loProviderExt.org = ioProvider.org;
        loProviderExt.org_guid = ioProvider.org_guid;
        loProviderExt.space = ioProvider.space;
        loProviderExt.space_guid = ioProvider.space_guid;
        loProviderExt.name = ioProvider.name;
        loProviderExt.url = ioProvider.url + '/' + lsExtensionRoute;
        loProviderExt.extention = lsExtensionName;
        loProviderExt.route = lsExtensionRoute;
      }
    });
  });
  if (typeof loProviderExt.url === 'undefined' || loProviderExt.url === null) {
    throw new Error('Failed, because was not able to construct the url for the service provider for ' + isExtensionFile + ' files');
  }
  return loProviderExt;
};
const cfTaskChainPostBatch = function(isExt, iaoTaskChainPostBatch, iaoResultItem, ioProviderExt, ioCredential) {

  let laoObjectInstanceToDelete = [];
  let laoResultItem = iaoResultItem.get_aoItem(isExt);
  laoResultItem.forEach(function(ioResultItem) {
    if(ioResultItem[Result.csKey_AOI_Action] === Result.ACTIONS.DELETE) {
        let loObjectInstance = {
      'namespace': ioResultItem[Result.csKey_AOI_sNamespace],
      'taskChainId': ioResultItem[Result.csKey_AOI_nID],
      'clear': true
    };
    laoObjectInstanceToDelete.push(loObjectInstance);
    }
  });

  let lsProviderUrl = ioProviderExt.url;
  let loData = {
    uri: lsProviderUrl,
    json: true,
    method: 'POST',
    auth: {
      user: ioCredential.user,
      password: ioCredential.password,
      sendImmediately: true
    },
    body: iaoTaskChainPostBatch
  };
  let lfTask = function (ifCb) {
    coLoggerLib.trace(' * Test ' + isExt + ' - ' + loData.uri + ' ' + loData.method + ' ... ');
    let lfcb = coRepositoryLib.lfCb2Reuse(isExt, loData, ifCb );
    coRequestLib(loData, lfcb);
  };
  let lafTask = [];
  lafTask.push(lfTask);
    let loData4Delete = {
    uri: lsProviderUrl,
    json: true,
    method: 'DELETE',
    auth: {
      user: ioCredential.user,
      password: ioCredential.password,
      sendImmediately: true
    },
    body: laoObjectInstanceToDelete
  };
  let lfTaskDelete = function (ifCb) {
    coLoggerLib.trace(' * ' + isExt + ' - ' + loData4Delete.uri + ' ' + loData4Delete.method + ' ... ');
    let lfcb = coRepositoryLib.lfCb2Reuse(isExt, loData4Delete, ifCb );
    coRequestLib(loData4Delete, lfcb);
  };
   lafTask.push(lfTaskDelete);
  return lafTask;
};

const cfTaskChainPostRepo = function(isExt, iaoResultItem, ioProviderExt, ioCredential, ioOption) {
  let loRepository = new coRepositoryLib.Repository(ioCredential, ioOption);
  let lafTaskWaterfall = loRepository.get_afTask2Update(isExt, iaoResultItem, ioProviderExt);
  let lafTask = [];
  lafTask.push(function(ifCb) {
    coAsyncLib.waterfall(lafTaskWaterfall, function(ioError, ioResult) {
      ifCb(ioError, ioResult);
    });
  });
  return lafTask;
};

const coType = {};
coType[coAbstractLib.csExtention] = '.dwftaskchain';
coType[coAbstractLib.cfPrepare] = undefined;
coType[coAbstractLib.cfCreateTasks] = function(iaoTaskChainPostBatch, ioResult, ioCredential, ioOption) {
  coValidateLib(iaoTaskChainPostBatch.length, 'number');
  coValidateLib(ioResult, 'object');
  coValidateLib(ioCredential, 'object');
  coValidateLib(ioOption, 'object');

  let loResult = ioResult;
  let lsExt = coType[coAbstractLib.csExtention];

  // Creates the Task Chain for the Batch Post
  let laoTaskChainPostBatch = iaoTaskChainPostBatch;//coContainerLib.cfCalculateHash(iasFile, ioPreparedValue4Ext, ioCredential, ioOption);
  // Greates tasks for TOE and Repo
  let loProviderExt = cfGetProvider(ioCredential[coDwfAbstract.cs_Credential_Key_Providers_v1], lsExt);
  let loValue_afTask = [];
  let lafTaskToe = cfTaskChainPostBatch(lsExt, laoTaskChainPostBatch, loResult, loProviderExt, ioCredential);
  loValue_afTask = loValue_afTask.concat(lafTaskToe);
  let lafTaskRepo = cfTaskChainPostRepo(lsExt, loResult.get_aoItem(lsExt), loProviderExt, ioCredential, ioOption);
  loValue_afTask = loValue_afTask.concat(lafTaskRepo);
  loResult.addValue(lsExt, loValue_afTask);

  return loResult;
};

module.exports = coType;
